
/**************************************************************

File Name:		matrixbyvector.asm
Based on: 		mxnxnx1.asm 4/25/91 SC	
Date Modified:	2/10/00 GGL
				6/6/00	GGL	modified so that SIMD is enabled in
							matrixbyvector.asm instead of matrix_test.asm
				6/7/00	GGL verified on 0.1 Si; won't work in 0.0
				9/19/04	JDT	Ported to VDSP++ 3.5

Description:	Multiplication of a (MxN) matrix by a (Nx1) vector

Calling Parameters:
			b0,i0 = address matrix A
			l0 = length of matrix A
			b1,i1 = address of matrix C
			b8,i8 = address of matrix B
			l8 = length of matrix B
			r15 = number of columns
			r14 = number of rows divided by 2
			m0 = 1
			m1,m9 = 2

Assumptions:
			All arrays must start on even address boundaries.
			
Return Values:
			i8 = mat_c 

Registers Affected:
			f0,s0,f4,s4,f8,s8,f12,s12
			i0,i1,i8

Cycle Count:
			[M * (N/2 + 4)] + 8 + 5 cache misses

Number of PM Locations:
			14 instrucation locations
			mat_b buffer
						
Number of DM Locations:
			mat_a buffer
			mat_c buffer

Notes:		All the matrices are stored in row major order. 
			N and M must be even numbers.  Matrix arrays 
			must be of even length (zero pad if necessary.) 
			The last value in the output matrix (mat_c) is a dummy value. 

Silicon Revision Issue:
			This code will not function properly on silicon revision 0.0 
			due to an anomaly related to internal addressing in SIMD mode.
			For more information please refer to the anomaly list.

Circular Buffering Issue:
			Because SIMD or Long word access transfer two 32-bit words, programs
			must be careful when using these accesses in circular buffering.
			It is important that SIMD or Long word accesses do not cross a circular
		 	buffer boundary.  If a SIMD	mode access occurs using a circular buffer
			index register that points to the last location in the circular buffer
		 	(end of buffer), the resulting access transfers the last location in
		 	the circular buffer and the	first location outside the buffer (end of buffer + 1).

**************************************************************/

#include    "def21364.h"

.global mxnxnx1;

/* Program memory code */
.section/PM seg_pmco;

mxnxnx1:	bit set MODE1 PEYEN | CBUFEN; /* enable SIMD mode, circular buffers */
nop; /*1 cycle latency before SIMD is enabled */
r8=r8 xor r8, f0=dm(i0,m1), f4=pm(i8,m9); 	/* clear f8, read a SIMD pair of samples from mat_A and mat_B */
f12=f0*f4,   f0=dm(i0,m1), f4=pm(i8,m9); /* calc mat_A * mat_B, read next SIMD pair from mat_A and mat_B */
	lcntr=r15, do column until lce;
		lcntr=r14, do row until lce;
row:		f12=f0*f4, f8=f8+f12, f0=dm(i0,m1), f4=pm(i8,m9); /*calc mat_A * mat_B,accumulate,read mat_A and mat_B */
		r9<->s8; /*move Pey to Pex*/
		f8=f8+f9;	/*add values*/
column:		r8=r8 xor r8, dm(i1,m0)=r8;	/*save values in mat_c, clear r8*/
		rts(db);
		bit clr mode1 PEYEN | CBUFEN;	/*disable SIMD and circular buffers after next cycle*/
		dm(i1,m0)=0;	/*the last value in the buffer is a dummy value, here we clear it*/

